from enum import Enum
from typing import Final
from typing import Optional, Any, Callable, Union

from .independent.helpful import toBitSize
from .independent.log_file import LogFile
from .independent.zip_file import ZippFile, ZipCompression

class MetaLogger:
    """
    Мета данные логгера
    """
    #: Конец цвета
    reset_: Final[str] = "\x1b[0m"
    blue: Final[str] = "\x1b[96m"
    yellow: Final[str] = "\x1b[93m"
    read: Final[str] = "\x1b[91m"
    green: Final[str] = "\x1b[92m"
    #: Серый
    gray: Final[str] = "\x1b[90m"
    #: Неон
    neon: Final[str] = "\x1b[96m"


class CompressionLog(Enum):
    """
    Варианты действий при достижении лимита размера файла
    """
    #: Перезаписать файл (Удалить все и начать с 0)
    rewrite_file = lambda _path_file: CompressionLog._rewrite_file(_path_file)

    #: Сжать лог файл в архив, а после удалить лог файл
    zip_file = lambda _path_file: CompressionLog._zip_file(_path_file)

    @staticmethod
    def _rewrite_file(_path_file: str):
        _f = LogFile(_path_file)
        logger.system_info(f"{_path_file}:{_f.sizeFile()}", flag="DELETE")
        _f.deleteFile()

    @staticmethod
    def _zip_file(_path_file: str):
        ZippFile(f"{_path_file}.zip").writeFile(_path_file, compression=ZipCompression.ZIP_LZMA)
        LogFile(_path_file).deleteFile()
        logger.system_info(_path_file, flag="ZIP_AND_DELETE")


class loglevel:
    """
    Создание логгера
    """
    __slots__ = [
        "level",
        "fileout",
        "console_out",
        "color_flag",
        "color_loglevel",
        "max_size_file",
        "compression",
        "_cont_write_log_file",
    ]

    #: Через сколько записей в лог файл, проверять его размер.
    CONT_CHECK_SIZE_LOG_FILE = 10

    def __init__(
            self, level: str,
            fileout: Optional[str] = None,
            console_out: bool = True,
            color_flag: str = "",
            color_loglevel: str = "",
            max_size_file: Optional[Union[int, str]] = "10mb",
            compression: Optional[Union[CompressionLog, Callable]] = None,
    ):
        """
        Создать логгер

        :param level: Уровень
        :param fileout: Куда записать данные
        :param console_out: Нужно ли выводить данные в ``stdout``
        :param max_size_file: Максимальный размер(байтах), файла после которого происходит ``compression``.

        Также можно указать:

        - kb - Например 10kb
        - mb - Например 1mb
        - None - Без ограничений

        :param compression: Что делать с файлам после достижение ``max_size_file``
        """
        self.level: str = level
        self.fileout: Optional[str] = fileout
        self.console_out: bool = console_out
        self.color_flag: str = color_flag
        self.color_loglevel: str = color_loglevel
        self.max_size_file: Optional[int] = toBitSize(max_size_file) if max_size_file else None
        self.compression: Callable = compression if compression else CompressionLog.rewrite_file

        #: Сколько раз было записей в лог файл, до выполнения
        #: условия ``self._cont_write_log_file < CONT_CHECK_SIZE_LOG_FILE``
        self._cont_write_log_file = 0

    def __call__(self, data: str, flag: str = ""):
        """
        Вызвать логгер

        :param data:
        :param flag:
        :return:
        """
        self._base(data, flag)

    def _base(self, data: Any, flag: str):
        """
        Логика работы логера

        :param data:
        :param flag:
        :return:
        """
        if self.fileout:
            log_formatted = "{level}[{flag}]:{data}\n".format(
                level=self.level,
                flag=flag,
                data=data,
            )
            _f = LogFile(self.fileout)
            _f.appendFile(log_formatted)
            # Проверить размер файла
            self._check_size_log_file(_f)

        if self.console_out:
            log_formatted = "{color_loglevel}{level}{reset}{color_flag}[{flag}]{reset}:".format(
                level=self.level,
                color_loglevel=self.color_loglevel,
                reset=MetaLogger.reset_,
                flag=flag,
                color_flag=self.color_flag
            )
            print(f"{log_formatted}{data}")

    def _check_size_log_file(self, _file: LogFile):
        """
        Проверить размер файла при достижении условия определенного
        количества записи в файл

        :param _file: Файл
        """
        if self._cont_write_log_file > self.CONT_CHECK_SIZE_LOG_FILE or self._cont_write_log_file == 0:
            self._check_compression_log_file(size_file=_file.sizeFile())
        self._cont_write_log_file += 1

    def _check_compression_log_file(self, size_file: int):
        """
        Проверить нужно ли выполнять  ``compression``

        :param size_file: Размер файла в байтах
        """
        if self.max_size_file is not None:
            if size_file > self.max_size_file:
                self.compression(self.fileout)


class logger:
    """
    Стандартные логгеры
    """

    info = loglevel(
        "[INFO]",
        color_loglevel=MetaLogger.blue,
        color_flag=MetaLogger.yellow,
    )
    error = loglevel(
        "[ERROR]",
        color_loglevel=MetaLogger.read,
        color_flag=MetaLogger.yellow,
    )
    success = loglevel(
        "[SUCCESS]",
        color_loglevel=MetaLogger.green,
        color_flag=MetaLogger.gray,
    )

    #: Логгер для системных задач
    system_info: Final[loglevel] = loglevel(
        "[SYSTEM]",
        color_loglevel=MetaLogger.gray,
        color_flag=MetaLogger.gray,
        console_out=True
    )
    #: Логгер для системных задач
    system_error: Final[loglevel] = loglevel(
        "[SYSTEM]",
        color_loglevel=MetaLogger.gray,
        color_flag=MetaLogger.read,
        console_out=True
    )



import re
from typing import Union


def toBitSize(size: Union[str, int]) -> int:
    """
    :param size:
    можно указать:
        - kb - Например 10kb
        - mb - Например 1mb

    >>> toBitSize("10kb")
    10240
    >>> toBitSize("1mb")
    1048576
    >>> toBitSize("1gb")
    Traceback (most recent call last):
    ValueError: Не верный тип 1gb
    """
    match size:
        case int():
            return size
        case str() as _res if _r := re.match("([\d_]+)kb|KB", _res):
            return int(_r.group(1)) * 1024
        case str() as _res if _r := re.match("([\d_]+)mb|MB", _res):
            return int(_r.group(1)) * 1048576
        case _:
            raise ValueError(f"Не верный тип {size}")



from abc import abstractmethod
from os import makedirs, remove, mkdir
from os.path import abspath, dirname, exists, getsize, splitext
from shutil import rmtree
from typing import Any, Union, Optional
from typing import Dict, Literal


class BaseFile:
    """
    Базовый класс для файлов
    """
    __slots__ = "name_file"

    def __init__(self, name_file: str, type_file: Optional[str]):
        """
        :param type_file: Какое расширение должен иметь файл. Если указать
            None то это значит что проверка попуститься
        :param name_file: Путь к файлу
        """
        self.check_extensions_file(name_file, type_file)
        self.name_file: str = name_file  #: Путь к файлу
        self.createFileIfDoesntExist()

    @staticmethod
    def check_extensions_file(name_file: str, req_type: str):
        """
        Проверить расширение файла

        :param name_file: Путь к файлу
        :param req_type: Требуемое расширение
        """
        # Если не нужно проверять имя расширения
        if not req_type:
            return
        if splitext(name_file)[1] != req_type:  # Проверяем расширение файла
            raise ValueError(f"Файл должен иметь расширение {req_type}")

    def createFileIfDoesntExist(self):
        """
        Создать файл если его нет
        """
        if not exists(self.name_file):
            tmp_ = dirname(self.name_file)
            if tmp_ and not exists(tmp_):  # Если задан путь из папок если их нет
                makedirs(tmp_)  # Создаем путь из папок
                open(self.name_file, "w").close()
            else:
                # Если указано только имя файла без папок
                # или папки же существуют
                open(self.name_file, "w").close()

    def checkExistenceFile(self) -> bool:  # +
        """
        Проверить существование файла
        """
        return True if exists(self.name_file) else False

    def deleteFile(self):  # +
        """
        Удалить файл
        """
        # Удаление файла
        if self.checkExistenceFile():
            remove(self.route())

    def sizeFile(self) -> int:  # +
        """
        Получить размер файла
        """
        # Размер файла в байтах
        return getsize(self.name_file)

    def route(self) -> str:  # +
        """
        Получить абсолютный путь
        """
        # Путь к файлу
        return abspath(self.name_file)

    def createRoute(self):
        """
        Создать файл по указному пути, если нужно поместить в папки, то они создадутся
        """
        tmp_route: str = ""
        for folder_name in self.name_file.split('/')[:-1]:
            tmp_route += folder_name
            mkdir(tmp_route)
            tmp_route += '/'

    def removeRoute(self):
        """
        Удалить весь путь к файлу
        """
        rmtree(self.name_file.split('/')[1])

    @abstractmethod
    def readFile(self, *arg) -> Any:
        """
        Прочитать файл
        """
        ...

    @abstractmethod
    def writeFile(self, arg: Any):
        """
       Записать данные в файл
       """
        ...

    @abstractmethod
    def appendFile(self, arg: Any):
        """
        Добавить данные в файл
        """
        ...


class TxtFile(BaseFile):
    """
    Работа с текстовым файлом

    :Example:

    .. code-block:: python

        from  mg_file.file.txt_file import TxtFile
        txt_obj = TxtFile('./path/file.txt')
    """

    def __init__(self, name_file: str, *,
                 mod: Optional[Literal['r', 'w', 'rb', 'wb', 'a', 'ab']] = None,
                 encoding: str = None,
                 data: Any = None,
                 type_file: Optional[str] = ".txt"):
        """
        Вы можете сразу выполнить метод указав ``mod``

        :param name_file:
        :param mod:
        :param encoding:
        :param data:
        :param type_file: Какое расширение должен иметь файл
        """
        super().__init__(name_file, type_file)
        if mod:
            self.res: Union[str, Any] = {
                "r": lambda: self.readFile(encoding=encoding),
                "w": lambda: self.writeFile(data=data),
                "rb": lambda: self.readBinaryFile(),
                "wb": lambda: self.writeBinaryFile(data=data),
                "a": lambda: self.appendFile(data=data),
                "ab": lambda: self.appendBinaryFile(data=data)
            }[mod]()

    def readFile(self, limit: int = 0, *, encoding: str = None) -> str:
        """
        Прочитать файл с начало

        :param limit: Ограничение чтения строк
        :param encoding: Кодировка
        """
        with open(self.name_file, "r", encoding=encoding) as f:
            if limit:
                res: str = ""
                for line in f:
                    if limit:
                        res += line
                        limit -= 1
                    else:
                        break
                return res
            else:
                return f.read()

    def readFileToResDict(self, *args: str, separator: str = '\n') -> Dict[str, str]:
        """
        Прочитать файл и вернуть словарь

        :param separator: Разделитель
        :param args: Имя ключей словаря

        :Пример:

        Файл ``./path/file.txt``

        .. code-block:: txt

            denisxab
            denis-k@mail.com
            password123

        Код

        .. code-block:: python

            from  mg_file.file.txt_file import TxtFile
            TxtFile('./path/file.txt').readFileToResDict("user_name","email","password")
            # {'user_name': 'denisxab', 'email': 'denis-k@mail.com', 'password': 'password123'}
        """
        resDict: Dict[str, str] = {}
        with open(self.name_file, "r") as f:
            for index, line in enumerate(f):
                resDict[args[index]] = line.replace(separator, "")
        return resDict

    def search(self, name_find: str) -> bool:
        """
        Простой поиск на соответствие тексту в файле

        :param name_find: Что искать

        :Пример:

        Файл ``./path/file.txt``

        .. code-block:: txt

            Optional. If the number of
            bytes returned exceed the hint number,
            no more lines will be returned. Default value is  -1,
            which means all lines will be returned.

        Код

        .. code-block:: python

            from  mg_file.file.txt_file import TxtFile
            TxtFile('./path/file.txt').searchFile("Default")
            # True
            TxtFile('./path/file.txt').searchFile("БУКВА")
            # False
        """
        res: bool = False
        with open(self.name_file, "r") as f:
            for line in f:
                if line.find(name_find) != -1:
                    res = True
                    break
        return res

    def readBinaryFile(self) -> bytes:  # +
        """
        Прочитать файл в бинарном режиме
        """
        with open(self.name_file, "rb") as f:
            return f.read()

    def writeFile(self, data: str):
        with open(self.name_file, "w") as f:
            f.write(data)

    def writeBinaryFile(self, data: Union[bytes, memoryview]):
        """
        Записать данные в бинарном режиме
        """
        with open(self.name_file, "wb") as f:
            f.write(data)

    def appendFile(self, data: str):  # +
        with open(self.name_file, "a") as f:
            f.write(data)

    def appendBinaryFile(self, data: bytes):
        """
        Добавить данные в бинарном режиме
        """
        with open(self.name_file, "ab") as f:
            f.write(data)


class LogFile(TxtFile):
    """
    Класс дял работы с лог файлами
    """

    def __init__(self, name_file: str, *, mod: Optional[Literal['r', 'w', 'rb', 'wb', 'a', 'ab']] = None,
                 encoding: str = None, data: Any = None, type_file: Optional[str] = ".log"):
        super().__init__(name_file, mod=mod, encoding=encoding, data=data, type_file=type_file)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logsmal', 'logsmal.independent']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'logsmal',
    'version': '0.0.5',
    'description': 'Создание файлов конфигураци',
    'long_description': '## Использование\n\n```python\nfrom logsmal import logger\n\nlogger.success("Программа запущена", flag="RUN")\n```\n\nСоздать кастомный логгер. Посмотрите все доступные аргменты :meth:`logsmal.loglevel.__init__()`\n\n```python\nfrom logsmal import loglevel, logger, CompressionLog\n\nlogger.MyLogger = loglevel(\n    level="[melogger]",\n    fileout="./log/mylog.log",\n    max_size_file="10kb",\n    console_out=False,\n    compression=CompressionLog.zip_file\n)\n```',
    'author': 'Denis Kustov',
    'author_email': 'denis-kustov@rambler.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/denisxab/logsmal.git',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from enum import Enum
from os import path, listdir
from typing import Optional, Any, Callable
from zipfile import ZIP_LZMA, ZipFile, ZIP_DEFLATED, ZIP_BZIP2, ZIP_STORED

from .log_file import BaseFile


class ZipCompression(Enum):
    """
    Степень сжатия, чем ниже тем больше
    """
    ZIP_STORED = ZIP_STORED
    ZIP_DEFLATED = ZIP_DEFLATED
    ZIP_BZIP2 = ZIP_BZIP2
    ZIP_LZMA = ZIP_LZMA


class ZippFile(BaseFile):
    """
    Работа с архивом
    """

    def __init__(
            self, name_file: str, type_file: Optional[str] = ".zip",
            call_log_info: Callable[[str, str], None] = lambda _x, flag: ...,
            call_log_error: Callable[[str, str], None] = lambda _x, flag: ...,
    ):
        super().__init__(name_file, type_file=type_file)
        self.call_log_info: Callable[[str, str], None] = call_log_info
        self.call_log_error: Callable[[str, str], None] = call_log_error

    def writeFile(self, in_path: str, compression: ZipCompression = ZipCompression.ZIP_DEFLATED, arcname=None):
        """
        Записать файл в архив

        :param in_path: Путь к файлу
        :param compression: Степень сжатия файла
        :param arcname: По какому пути будет расположен файл в архиве
        """
        with ZipFile(self.name_file, 'w', compression=compression.value) as zip_file:
            if path.isfile(in_path):
                zip_file.write(in_path, arcname)


    def writePath(self,
                  in_path: str,
                  execute_path: Optional[set[str]] = None,
                  compression: ZipCompression = ZipCompression.ZIP_DEFLATED
                  ):

        """
        Заархивировать файлы и папки из указанного пути

        :param in_path: Путь к папке в которой нужно все за архивировать.
          Кроме того что указано в аргументе ``execute_path``
        :param execute_path: Что не архивировать
        :param compression: Сжатие файла
        """
        # Если исключений нет, то множеств пустое
        if execute_path is None:
            execute_path = set()
        # Если файл, то архивируем его, если папка, то рекурсивно
        # архивируем каждый файл
        with ZipFile(self.name_file, 'w', compression=compression.value) as zip_file:
            if path.isfile(in_path):
                zip_file.write(in_path)
            else:
                self._addFolderToZip(
                    zip_file,
                    in_path,
                    _base_folder=in_path,
                    execute_path=execute_path
                )

    def _addFolderToZip(
            self,
            zip_file: ZipFile,
            in_folder: str,
            execute_path: set[str],
            _base_folder: str
    ):
        """
        Рекурсивная архивация директории

        :param zip_file: Дескриптор архива
        :param in_folder: Текущая директория
        :param execute_path: Что не нужно архивировать
        :param _base_folder: Главный путь в котором происходит архивация
            именно его мы отсечем от абсолютно пути, и запишем относительный путь в архив
        """
        #: Имя текущего архива
        name_self_zip_file: str = path.basename(zip_file.filename)
        for file in listdir(in_folder):
            # Мы не должны архивировать свой же архив, и те что в исключении
            if file != name_self_zip_file and file not in execute_path:
                full_path = path.join(in_folder, file)
                if path.isfile(full_path):
                    zip_file.write(
                        full_path,
                        # Нам не нужен полный путь в архиве, поэтому мы его заменим на относительный
                        arcname=full_path.replace(_base_folder, '')
                    )
                    self.call_log_info(full_path, flag="FILE_ADD")
                elif path.isdir(full_path):
                    self._addFolderToZip(
                        zip_file,
                        full_path,
                        execute_path,
                        _base_folder
                    )
                    self.call_log_info(full_path, flag="FOLDER_ADD")

            else:
                self.call_log_info(file, flag="SKIP")

    def readFile(self, *arg) -> Any:
        pass

    def appendFile(self, arg: Any):
        pass
